require "memoist"
require "aws-sdk-s3"

module ContainersCp
  class S3Access
    include Import["logger", "gzip"]
    extend ::Memoist

    # For the future: This is nearly-identical code to the version in +cht-integration-aws+ and should be extracted
    # to +cht-framework-cp+.

    REGION = ENV['AWS_REGION'] || "us-east-1"

    def get_uncompressed(bucket, key, region: REGION, compression: :gz)
      content = get bucket, key, region: region
      case compression.to_sym
      when :gz
        gzip.uncompress content
      when :lz4
        ::LZ4.uncompress content
      else
        raise "Unsupported compression: #{compression}"
      end
    end

    def get(bucket, key, region: REGION)
      s = get_stream(bucket, key, region: region)
      s.read
    ensure
      s&.close
    end

  def get_stream(bucket, key, region: REGION)
      logger.debug "Download from #{bucket}:#{key}"
      s3_client(region) do |client|
        client.get_object(bucket: bucket, key: key).body
      end
    end

    private
    def s3_client(region)
      attempts = 0
      begin
        opts = aws_sdk_core_common_options(region)
        # Initializing a new S3 Client object will pull in credentials from EKS environment and automatically
        # refresh credentials periodically
        @s3_client ||= ::Aws::S3::Client.new opts
        yield @s3_client
      rescue ::Aws::S3::Errors::InvalidAccessKeyId
        attempts += 1
        raise if attempts > 5
        logger.warn "[S3Access] Aws::S3::Errors::InvalidAccessKeyId: Sleeping, and then retrying... (attempt #{attempts})"
        sleep attempts
        retry
      end
    end

    def aws_sdk_core_common_options(region)
      {
        force_path_style: true,
        region: region,

        user_agent_suffix: "cloudhealth-containers-cp-srv",
        retry_limit: 5,

        # Override defaults from aws-sdk-core-2.0.33/lib/seahorse/client/net_http/connection_pool.rb
        # Default for http_read_timeout is 60s, but we get timeouts for customers with massive data sets, like ec2 snapshots
        # Seahorse::Client::NetworkingError: Timeout::Error
        http_read_timeout: 5.minute.to_i,
        http_idle_timeout: 10
      }
    end
    memoize :aws_sdk_core_common_options
  end
end
